import boto3

from moto import mock_aws


@mock_aws
def test_destination_tags():
    logs = boto3.client("logs", "us-west-2")
    destination_name = "test-destination"
    role_arn = "arn:aws:iam::123456789012:role/my-subscription-role"
    target_arn = "arn:aws:kinesis:us-east-1:123456789012:stream/my-kinesis-stream"

    destination_arn = logs.put_destination(
        destinationName=destination_name,
        targetArn=target_arn,
        roleArn=role_arn,
        tags={"key1": "val1"},
    )["destination"]["arn"]

    _verify_tag_operations(destination_arn, logs)


@mock_aws
def test_log_groups_tags():
    logs = boto3.client("logs", "us-west-2")
    log_group_name = "test"

    logs.create_log_group(logGroupName=log_group_name, tags={"key1": "val1"})
    arn = logs.describe_log_groups()["logGroups"][0]["logGroupArn"]

    _verify_tag_operations(arn, logs)


def _verify_tag_operations(arn, logs):
    logs.tag_resource(resourceArn=arn, tags={"key2": "val2"})
    tags = logs.list_tags_for_resource(resourceArn=arn)["tags"]
    assert tags == {"key1": "val1", "key2": "val2"}
    logs.untag_resource(resourceArn=arn, tagKeys=["key2"])
    tags = logs.list_tags_for_resource(resourceArn=arn)["tags"]
    assert tags == {"key1": "val1"}
